@@ -5,11 +5,12 @@ from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Count, Min, Q
 from django.contrib.sessions.models import Session
 from django.utils import formats
-import os, re
+import os, re, time
 from datetime import datetime
+from signal import SIGABRT
 
 from django.contrib.auth.models import User
-from models import Song, Artist, Album, Genre, Queue, Favourite, History
+from models import Song, Artist, Album, Genre, Queue, Favourite, History, Player
 
 
 class api_base:
@@ -365,7 +366,7 @@ class songs(api_base):
                 self.addToHistory(song_instance, None)
 
         # remove missing files
-        if not os.path.exists(song_instance.Filename):
+        if not os.path.exists(song_instance.Filename.encode('utf8')):
             Song.objects.all().filter(id=song_instance.id).delete()
             return self.getNextSong()
 
@@ -376,7 +377,7 @@ class songs(api_base):
 
         # get logged in users
         sessions = Session.objects.exclude(
-            expire_date__gt=datetime.today()
+            expire_date__lt=datetime.today()
         )
         for session in sessions.all():
             data = session.get_decoded()
@@ -435,6 +436,14 @@ class songs(api_base):
             for user_instance in user_list.all():
                 history_instance.User.add(user_instance)
 
+    def skipCurrentSong(self):
+        players = Player.objects.all()
+        for player in players:
+            try:
+                os.kill(player.Pid, SIGABRT)
+            except OSError:
+                player.delete()
+
 class history(api_base):
     order_by_fields = {
         "title": "Song__Title",
@@ -519,6 +528,49 @@ class history(api_base):
 
         return result
 
+    def getCurrent(self):
+        item = History.objects.all()[0:1].get()
+        createdTimestamp = time.mktime(item.Created.timetuple())
+        dataset = {
+            "id": item.Song.id,
+            "title": None,
+            "artist": {
+                "id": None,
+                "name": None,
+            },
+            "album": {
+                "id": None,
+                "title": None,
+            },
+            "year": None,
+            "genre": {
+                "id": None,
+                "name": None,
+            },
+            "queued": False,
+            "favourite": False,
+            "created": formats.date_format(
+                item.Created, "DATETIME_FORMAT"
+            ),
+            "votes": item.User.count(),
+            "users": [],
+            "remaining": createdTimestamp + item.Song.Length - int(time.time())
+        }
+        if not item.Song.Title is None:
+            dataset["title"] = item.Song.Title
+        if not item.Song.Artist is None:
+            dataset["artist"]["id"] = item.Song.Artist.id
+            dataset["artist"]["name"] = item.Song.Artist.Name
+        if not item.Song.Album is None:
+            dataset["album"]["id"] = item.Song.Album.id
+            dataset["album"]["title"] = item.Song.Album.Title
+        if not item.Song.Year is None:
+            dataset["year"] = item.Song.Year
+        if not item.Song.Genre is None:
+            dataset["genre"]["id"] = item.Song.Genre.id
+            dataset["genre"]["name"] = item.Song.Genre.Name
+
+        return dataset
 
 class history_my(history):
     order_by_fields = {
@@ -907,3 +959,20 @@ class years(api_base):
             result["itemList"].append(dataset)
 
         return result
+
+
+class players(api_base):
+    def add(self, pid):
+        player = Player(
+            Pid=pid
+        )
+        player.save()
+
+        return player.id
+
+    def remove(self, pid):
+        Player.objects.get(Pid=pid).delete()
+
+        return {
+            "pid": pid,
+        }
